/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkInformationStringVectorKey.h

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkInformationStringVectorKey - Key for String vector values.
// .SECTION Description
// vtkInformationStringVectorKey is used to represent keys for String
// vector values in vtkInformation.h

#ifndef __vtkInformationStringVectorKey_h
#define __vtkInformationStringVectorKey_h

#include "vtkInformationKey.h"

#include "vtkCommonInformationKeyManager.h" // Manage instances of this type.

class VTK_COMMON_EXPORT vtkInformationStringVectorKey : public vtkInformationKey
{
public:
  vtkTypeRevisionMacro(vtkInformationStringVectorKey,vtkInformationKey);
  void PrintSelf(ostream& os, vtkIndent indent);

  vtkInformationStringVectorKey(const char* name, const char* location,
                                 int length=-1);
  ~vtkInformationStringVectorKey();

  // Description:
  // Get/Set the value associated with this key in the given
  // information object.
  void Append(vtkInformation* info, const char* value);
  void Set(vtkInformation* info, const char* value, int idx = 0);
  const char* Get(vtkInformation* info, int idx = 0);
  int Length(vtkInformation* info);
  int Has(vtkInformation* info);
  
  // Description:
  // Copy the entry associated with this key from one information
  // object to another.  If there is no entry in the first information
  // object for this key, the value is removed from the second.
  virtual void ShallowCopy(vtkInformation* from, vtkInformation* to);

  // Description:
  // Print the key's value in an information object to a stream.
  virtual void Print(ostream& os, vtkInformation* info);

protected:
  // The required length of the vector value (-1 is no restriction).
  int RequiredLength;

private:
  vtkInformationStringVectorKey(const vtkInformationStringVectorKey&);  // Not implemented.
  void operator=(const vtkInformationStringVectorKey&);  // Not implemented.
};

#endif
